#include <stdlib.h>

#include "buffer.h"

Buffer *
new_buffer (int initial_size)
{
    Buffer *b = malloc (sizeof (Buffer));
    
    b->buffer = malloc (initial_size);
    b->length = initial_size;
    memset (b->buffer, 0, b->length);
    b->index = -1;
}

Buffer *
buffer_dup (Buffer *b)
{
    Buffer *dup = new_buffer (b->length); 

    add (dup, b);

    return dup;
}

void
free_buffer (Buffer *b)
{
    free (b->buffer);
    free (b);
}

void
clear_buffer (Buffer *b)
{
    b->index = -1;
    memset (b->buffer, 0, b->length);
}

void
push_char (Buffer *b, int c)
{
    if (b->index == b->length-1) {
	b->length *= 2;
	b->buffer = realloc (b->buffer, b->length);
    }

    b->index++;
    b->buffer[b->index] = c;
}

int
pop_char (Buffer *b)
{
    int c = b->buffer[b->index];

    b->index--;

    return c;
}

void
remove_char (Buffer *b, int i)
{
    memmove (b->buffer+i, b->buffer+i+1, b->index+1-i);
    b->index--;
}

Buffer *
intersect (Buffer *a, Buffer *b)
{
    int i, j;
    Buffer *c;
    Buffer *temp = buffer_dup (b);

    c = new_buffer (8);
    
    for (i = 0; i <= a->index; i++) {
	for (j = 0; j <= temp->index; j++) {
	    if (index (a, i) == index (temp, j))  {
		push_char (c, index (a, i));
		remove_char (temp, j);
		break;
	    }
	}
    }

    free_buffer (temp);

    return c;
}

/* asumming a and b are disjoint */
void
add (Buffer *a, Buffer *b)
{
    int i;

    for (i = 0; i <= b->index; i++) {
	push_char (a, index (b, i));
    }
}

void
subtract (Buffer *a, Buffer *b)
{
    int i, j;
    Buffer *c = buffer_dup (b);

    for (i = 0; i <= a->index; i++) {
	for (j = 0; j <= c->index; j++) {
	    if (index (a, i) == index (c, j))  {
		remove_char (a, i);
		remove_char (c, j);
		i--;
		break;
	    }
	}
    }

    free_buffer(c);
}
