#include <stdio.h>

#include "tagged_text.h"
#include "output.h"

void
output_text_list (TaggedTextList *text_list)
{
    unsigned int i;
    TaggedText *cur;

    for (cur = text_list->head; cur != NULL; cur = cur->next) {
	if (cur->state.b)
	    printf (" B");
	if (cur->state.em)
	    printf (" EM");
	if (cur->state.i)
	    printf (" I");
	if (cur->state.s)
	    printf (" S");
	if (cur->state.tt)
	    printf (" TT");
	for (i = 0; i < cur->state.u; i++)
	    printf (" U");
	if (cur->state.size != DEFAULT_SIZE)
	    printf (" %d", cur->state.size);
	switch (cur->state.color) {
	case 0: printf ("r"); break;
	case 1: printf ("g"); break;
	case 2: printf ("b"); break;
	case 3: printf ("c"); break;
	case 4: printf ("m"); break;
	case 5: printf ("y"); break;
	case 6: printf ("k"); break;
	case 7: printf ("w"); break;
	}
	printf ("\n%s\n", cur->text);
    }
}

static void
output_ends (Block *block, unsigned int n)
{
    if (n == 0 || block == NULL)
	return;
 
    for (;n != 0 && !empty(block->stack); n--) {
	switch (pop_char (block->stack)) {
	case 0: printf ("</0>"); break;
	case 1: printf ("</1>"); break;
	case 2: printf ("</2>"); break;
	case 3: printf ("</3>"); break;
	case 4: printf ("</4>"); break;
	case 5: printf ("</5>"); break;
	case 6: printf ("</6>"); break;
	case 7: printf ("</7>"); break;
	case 8: printf ("</8>"); break;
	case 9: printf ("</9>"); break;
	case B: printf ("</B>"); break;
	case EM: printf ("</EM>"); break;
	case I: printf ("</I>"); break;
	case PL: printf ("</PL>"); break;
	case S: printf ("</S>"); break;
	case TT: printf ("</TT>"); break;
	case U: printf ("</U>"); break;
	case cr: printf ("</r>"); break;
	case cg: printf ("</g>"); break;
	case cb: printf ("</b>"); break;
	case cc: printf ("</c>"); break;
	case cm: printf ("</m>"); break;
	case cy: printf ("</y>"); break;
	case ck: printf ("</k>"); break;
	case cw: printf ("</w>"); break;	
	}
    }

    output_ends (block->prev, n);
}

static void
output_stack (Buffer *stack)
{
    int i;

    if (stack == NULL)
	return;

    for (i = 0; i <= stack->index; i++) {
	switch (stack->buffer[i]) {
	case 0: printf ("<0>"); break;
	case 1: printf ("<1>"); break;
	case 2: printf ("<2>"); break;
	case 3: printf ("<3>"); break;
	case 4: printf ("<4>"); break;
	case 5: printf ("<5>"); break;
	case 6: printf ("<6>"); break;
	case 7: printf ("<7>"); break;
	case 8: printf ("<8>"); break;
	case 9: printf ("<9>"); break;
	case B: printf ("<B>"); break;
	case EM: printf ("<EM>"); break;
	case I: printf ("<I>"); break;
	case PL: printf ("<PL>"); break;
	case S: printf ("<S>"); break;
	case TT: printf ("<TT>"); break;
	case U: printf ("<U>"); break;
	case cr: printf ("<r>"); break;
	case cg: printf ("<g>"); break;
	case cb: printf ("<b>"); break;
	case cc: printf ("<c>"); break;
	case cm: printf ("<m>"); break;
	case cy: printf ("<y>"); break;
	case ck: printf ("<k>"); break;
	case cw: printf ("<w>"); break;	
	}
    } 
}

void
output (Block *block_list)
{
    Block *prev = NULL, *cur, *next;

    for (cur = block_list; cur != NULL; cur = cur->next) {
	output_ends (prev, cur->n_pop);
	output_stack (cur->stack);
	printf (cur->text);
	prev = cur;
    }

    output_ends (prev, 0xffffffff);
    fflush (stdout);

    for (cur = block_list; cur != NULL; cur = next) {
	next = cur->next;
	free_buffer (cur->stack);
	free (cur);
    }
}
