#include <stdio.h>
#include <string.h>

#include "parse.h"
#include "output.h"

#define SPC 0x20
#define CR 0x0D
#define LF 0x0A
#define TAB 0x09
#define is_ws(c) (c == SPC || c == CR || c == LF || c == TAB)

State
compute_ws_context (State state)
{
    State ws_context = INITIAL_STATE;

    ws_context.tt = state.tt;
    ws_context.u = state.u;
    ws_context.size = state.size;
    if (ws_context.u == 0)
	ws_context.color = WHITE;
    else
	ws_context.color = state.color;

    return ws_context;
}

TaggedTextList *
parse ()
{
    int c;
    Buffer *text = new_buffer (1024);
    char tag[8]; /* for safety */
    int tag_index;
    int lexer_state = 0;
    int previous_ws = 0;
    State previous_ws_context = {0};
    State ws_context = {0};
    State previous_state;
    State state = INITIAL_STATE;
    StateStack *state_stack;
    TaggedTextList *text_list;

    memset (text_list, 0, sizeof(TaggedTextList));
    state_stack = push_state (NULL, state);

    /* parses into tagged text list */
    /* also removes excess white space */
    while ((c = getchar ()) != EOF) {
	switch (lexer_state) {
	case 0:
	    switch (c) {
	    case '<':
		lexer_state = 1;
		tag_index = 0;
		if (!empty (text)) {
		    if (text_list->tail
			&& state_equals(previous_state, state)) {
			text_list->tail->text = strcat (text_list->tail->text, text->buffer);
		    } else if (text_list->tail && strcmp (text->buffer, " ") == 0) {
			previous_ws_context = compute_ws_context (previous_state);
			ws_context = compute_ws_context (state);

			if (state_equals (previous_ws_context, ws_context))
			    text_list->tail->text = strcat (text_list->tail->text, " ");
			else
			    append_text (text_list, text, top_state (state_stack));
		    } else {
			append_text (text_list, text, top_state (state_stack));
		    }
		    previous_state = state;
		    clear_buffer (text);
		}
		break;
	    case SPC:
	    case CR:
	    case LF:
	    case TAB:
		ws_context = compute_ws_context (state);

		if (tt_set (state_stack)
		    || (!previous_ws || !state_equals(previous_ws_context, ws_context))) {
		    push_char (text, SPC);
		    previous_ws_context = ws_context;
		    previous_ws = 1;
		}
		break;
	    default:
		previous_ws = 0;
		push_char (text, c);
	    }
	    break;
	case 1:
	    switch (c) {
	    case '>':
		if (tag[0] == '/') {
		    state_stack = pop_state (state_stack);
		    state = state_stack->state;
		} else {
		    switch (tag[0]) {
		    case '0': state.size = 0; break;
		    case '1': state.size = 1; break;
		    case '2': state.size = 2; break;
		    case '3': state.size = 3; break;
		    case '4': state.size = 4; break;
		    case '5': state.size = 5; break;
		    case '6': state.size = 6; break;
		    case '7': state.size = 7; break;
		    case '8': state.size = 8; break;
		    case '9': state.size = 9; break;
		    case 'B': state.b = 1; break;
		    case 'E': state.em = !state.em; break;
		    case 'I': state.i = 1; break;
		    case 'P':
			state.b = 0;
			state.em = 0;
			state.i = 0;
			state.s = 0;
			state.tt = 0;
			state.u = 0;
			break;
		    case 'S': state.s = 1; break;
		    case 'T': state.tt = 1; break;
		    case 'U': state.u = state.u == 3? 3 : state.u+1; break;
		    case 'r': state.color = RED; break;
		    case 'g': state.color = GREEN; break;
		    case 'b': state.color = BLUE; break;
		    case 'c': state.color = CYAN; break;
		    case 'm': state.color = MAGENTA; break;
		    case 'y': state.color = YELLOW; break;
		    case 'k': state.color = BLACK; break;
		    case 'w': state.color = WHITE; break;
		    }
		    state_stack = push_state (state_stack, state);
		}
		lexer_state = 0;
		break;
	    case SPC:
	    case CR:
	    case LF:
	    case TAB:
		break;
	    default:
		tag[tag_index++] = c;
	    }
	    break;
	}
    }

    if (!empty (text))
	append_text (text_list, text, top_state (state_stack));

    free_buffer (text);

    /* optimize and output */
    /*output_text_list (text_list);*/
    return text_list;
}
